MODULE InterruptController!(SYSMODULE, VIEWONLY)

    VAR intnum timeint_control;
	VAR intnum timeint_log;
    
    VAR bool first_time := TRUE;
	
	VAR num position;
    VAR num position_int;
	VAR num velocity;
    
    VAR  num set_cur;
    
    VAR num approxFrom;
    VAR num approxTo;
    VAR clock approxClock;
    VAR num approxTime;
    VAR num lastApproxTime;
    VAR num lastPosition;
    VAR num posError;
    VAR num lastPosError;
    
    VAR num u;
    VAR num k_hat;
    VAR num k_hat_dot;
    VAR num k_hat_dot_last;
    VAR num sigma_hat;
    VAR num sigma_hat_dot;
    VAR num sigma_hat_dot_last;
    VAR num z;
    
    CONST num gamma_1 := 0.1;
    CONST num gamma_2 := 0.2;
    CONST num rho_1 := 15;
    CONST num rho_2 := 4;
    
        
	VAR num cur_bound := 0.4;    
	
    TRAP control_law
        IF start_flag =1 THEN
            IF(first_time) THEN           
                ClkStart approxClock;
                position := GripGetPos(GInputDnum(Grip_gi_Param1));
                velocity := GripGetVel(GInputDnum(Grip_gi_Param2));
                position_int := 0;
                lastApproxTime := 0;
                lastPosition := position;
                lastPosError := position - pos_ref;
                
                k_hat_dot_last := 0;
                sigma_hat_dot_last := 0;
            
                first_time := FALSE;
            ENDIF
            
            IF GInputDnum(Grip_gi_DLen) = 0x0f AND GInputDnum(Grip_gi_Cmd) = 0x95 THEN
                position := GripGetPos(GInputDnum(Grip_gi_Param1));
                velocity := GripGetVel(GInputDnum(Grip_gi_Param2));
            ELSE
                sendAck 0.01;
            ENDIF
            
            approxTime := ClkRead(approxClock);           
            posError := position - pos_ref;          
            position_int := position_int + trapezoidal_approx(lastPosError/1000, posError/1000, lastApproxTime ,approxTime);

            z := -rho_2*velocity/1000 - rho_1*(posError/1000) - position_int;
                 
			!IF k_hat < 4.5 THEN
                k_hat_dot := gamma_1*z*z;
            !ELSE
            !    k_hat_dot := 0;
            !ENDIF
                        
            
            !IF sigma_hat < 2.5 THEN
               sigma_hat_dot := gamma_2*z*calc_phi(z);
            !ELSE
            !    sigma_hat_dot := 0;
            !ENDIF
              
            k_hat := k_hat + trapezoidal_approx(k_hat_dot_last, k_hat_dot, lastApproxTime, approxTime);
            sigma_hat := sigma_hat + trapezoidal_approx(sigma_hat_dot_last, sigma_hat_dot, lastApproxTime, approxTime);
            
            u := k_hat*z + sigma_hat*calc_phi(z);
            
            set_cur := 0.01*u;

            
            IF set_cur > cur_bound  then set_cur := cur_bound; endif
		    IF set_cur < -cur_bound then set_cur := -cur_bound; endif
            
            GripMovCur set_cur, 0;

            
            k_hat_dot_last := k_hat_dot;
            sigma_hat_dot_last := sigma_hat_dot;
            lastPosition := position;
            lastApproxTime := approxTime;
            lastPosError := posError;
            
		ENDIF
    ENDTRAP
	
	TRAP log_file
		VAR num time;
	 IF log_start = 1 THEN
		time:=ClkRead(clock_prog);
		Write logfile, CTime()\NoNewLine;
		Write logfile, " "\Num:=time\NoNewLine;
		Write logfile, " "\Num:=position \NoNewLine;
		Write logfile, " "\Num:=velocity \NoNewLine;
        Write logfile, " "\Num:=position_int \NoNewLine;
		Write logfile, " "\Num:=u \NoNewLine;
        Write logfile, " "\Num:=k_hat \NoNewLine;
        Write logfile, " "\Num:=sigma_hat \NoNewLine;
		Write logfile, " # ";
	 ENDIF
	
	ENDTRAP
    
    FUNC num trapezoidal_approx(num f_a, num f_b, num timeA, num timeB)
        
        
        RETURN ((timeB - timeA)*(f_a + f_b)/2);
    ENDFUNC
    
    FUNC num calc_phi(num input)
        IF(input < 0) THEN
            RETURN -1;
        ELSEIF(input > 0) THEN
            RETURN 1;
        ELSE
            RETURN 0;
        ENDIF      
    ENDFUNC
        	
ENDMODULE